// HOW TO USE
// root [0] .L macros/examples/GenerateInputTree_SequentialDecay_28o.cc+
// root [1] GenerateInputTree_SequentialDecay_28o()

#include "TBeamSimData.hh"

#include "TFile.h"
#include "TTree.h"
#include "TH1.h"
#include "TH2.h"
#include "TGenPhaseSpace.h"
#include "TRandom3.h"
#include "TLorentzVector.h"
#include "TVector3.h"

#include <iostream>
#include <sstream>

void GenerateInputTree_SequentialDecay_28o(Double_t Erel_in=4.0, 
					   Double_t Erel_26o=0.02,// Erel for 26O, 20keV
					   Double_t E_in=200){
  // parameteres
  const int neve = 100000;

  cout<<"Erel_in = "<<Erel_in<<" MeV"<<endl
      <<"Erel_26o = "<<Erel_26o<<" MeV"<<endl
      <<"energy = "<<E_in<<" AMeV"<<endl
      <<"Num of events = "<<neve
      <<endl;


  const Int_t Abeam = 28;
  const Int_t Afrag = 24;
  const Int_t Zfrag = 8;
  const Int_t Ndecay = Abeam - Afrag + 1;

  const Int_t    Z[Ndecay]={Zfrag,0,0,0,0};
  const Int_t    A[Ndecay]={Afrag,1,1,1,1};
  const Double_t Masses[Ndecay] = {22370.8, 939.565, 939.565, 939.565, 939.565};
  const Double_t Mass_26o = Masses[0]+Masses[1]+Masses[2]+Erel_26o;
  const Double_t Masses_26onn[3] = {Mass_26o, Masses[3], Masses[4]};
  const Double_t Masses_24onn[3] = {Masses[0], Masses[1], Masses[2]};

  if (Erel_in<Erel_26o){
    cout<<"Invalid Erel inputs!"<<endl;
    return;
  }


  // Init Phase Space Decay
  Double_t Mbeam = Erel_in;
  for (Int_t i=0;i<Ndecay;++i) Mbeam+=Masses[i];
  Double_t Tbeam = Abeam*E_in;
  Double_t Ebeam = Mbeam + Tbeam;
  Double_t Pbeam_abs = sqrt(Tbeam*(Tbeam+2.0*Mbeam));
  TGenPhaseSpace Decay_26onn;
  TGenPhaseSpace Decay_24onn;

  stringstream ss;
  ss<<"root/storage/inputtree_28o_seqdecay_"<<Erel_in<<"_"<<Erel_26o<<".root";
  TFile *file = new TFile(ss.str().c_str(),"recreate");
  cout<<file->GetName()<<endl;
  TTree *tree = new TTree("tree_input","Input tree for simtrace");
  gBeamSimDataArray = new TBeamSimDataArray();
  tree->Branch("TBeamSimData", gBeamSimDataArray);

  // for check
  TH1* herel_gen = new TH1D("herel_gen","Erel generated",200,0,10);
  TH1* he012_e034_gen = new TH2D("he012_e034_gen","Erel012 Erel034 generated",100,0,10,100,0,10);
  TH1* he012_e034_gen_z = new TH2D("he012_e034_gen_z","Erel012 Erel034 generated (zoom)",100,0,0.1,100,0,10);
  TH1* hxypos = new TH2D("hxypos","XY pos (mm)",100,-50,50, 100,-50,50);
  TH1* hzxpos = new TH2D("hzxpos","ZX pos (mm)",100,-5000,-4000, 100,-50,50);
  TH1* hxyang = new TH2D("hxyang","XY ang (mard)",100,-50,50, 100,-50,50);

  for (int i=0;i<neve;++i){

    if (i%1000==0) 
      cout<<"\r "<<i<<" / "<<neve
	  <<" ("<<100.*i/neve<<"%)"
	  <<flush;

    gBeamSimDataArray->clear();

    TLorentzVector Pbeam(0, 0, Pbeam_abs, Ebeam);

    // position spread: uniform, +-20mm for XY
    Double_t Zpos_target = -4500.;
    //Double_t Zpos_target = -3867.28;// Dayone 
    TVector3 pos_vec(gRandom->Uniform(-20,20),
                     gRandom->Uniform(-20,20),
                     Zpos_target); 

    // angular spread xy: +-10mrad
    Double_t xang = gRandom->Gaus(0,0.01);
    Double_t yang = gRandom->Gaus(0,0.01);
    Pbeam.RotateY(xang);
    Pbeam.RotateX(-yang);
    Decay_26onn.SetDecay(Pbeam,3,Masses_26onn);

    // Generate decayed events
    double weight_26onn = Decay_26onn.Generate();
    while (gRandom->Uniform()>weight_26onn) weight_26onn = Decay_26onn.Generate();
    TLorentzVector P26o = *(Decay_26onn.GetDecay(0));

    Decay_24onn.SetDecay(P26o,3,Masses_24onn);
    double weight_24onn = Decay_24onn.Generate();
    while (gRandom->Uniform()>weight_24onn) weight_24onn = Decay_24onn.Generate();

    TLorentzVector P[Ndecay];
    P[0] = *(Decay_24onn.GetDecay(0));
    P[1] = *(Decay_24onn.GetDecay(1));
    P[2] = *(Decay_24onn.GetDecay(2));
    P[3] = *(Decay_26onn.GetDecay(1));
    P[4] = *(Decay_26onn.GetDecay(2));
    for (int i=0;i<Ndecay;++i){

      TBeamSimData particle(Z[i],A[i],P[i],pos_vec);
      if (Z[i]==0) particle.fParticleName="neutron";
      particle.fPrimaryParticleID = i;
      gBeamSimDataArray->push_back(particle);
    }
    
    // checking input
    TLorentzVector Ptot(0,0,0,0);
    Double_t Mtot=0;
    for (Int_t i=0;i<Ndecay;i++) {
      Ptot += P[i];
      Mtot += P[i].M();
    }
    Double_t Erel_gen = Ptot.M() - Mtot;
    herel_gen->Fill(Erel_gen);
    TLorentzVector P012(0,0,0,0);
    TLorentzVector P034(0,0,0,0);
    P012 = P[0]+P[1]+P[2];
    P034 = P[0]+P[3]+P[4];
    Double_t M012 = P[0].M()+P[1].M()+P[2].M();
    Double_t M034 = P[0].M()+P[3].M()+P[4].M();
    he012_e034_gen->Fill(P012.M()-M012, P034.M()-M034);
    he012_e034_gen_z->Fill(P012.M()-M012, P034.M()-M034);

    hxypos->Fill(pos_vec.x(), pos_vec.y());
    hzxpos->Fill(pos_vec.z(), pos_vec.x());
    hxyang->Fill(Ptot.Px()/Ptot.Pz()*1000., Ptot.Py()/Ptot.Pz()*1000.);

    // Fill tree
    tree->Fill();

  }
  file->Write();
}
